@@ -12,7 +12,6 @@ These are the defined components.
 # from python
 from types import ListType, TupleType
 SequenceTypes = (ListType, TupleType)
-import re
 
 # from this package
 from icalendar.caselessdict import CaselessDict
@@ -133,6 +132,12 @@ class Component(CaselessDict):
     >>> [i['dtstart'] for i in c.walk('VEVENT')]
     ['20000101T000000']
 
+    Text fields which span multiple mulitple lines require proper indenting
+    >>> c = Calendar()
+    >>> c['description']=u'Paragraph one\\n\\nParagraph two'
+    >>> c.as_string()
+    'BEGIN:VCALENDAR\\r\\nDESCRIPTION:Paragraph one\\r\\n \\r\\n Paragraph two\\r\\nEND:VCALENDAR\\r\\n'
+
     INLINE properties have their values on one property line. Note the double
     quoting of the value with a colon in it.
     >>> c = Calendar()
@@ -177,12 +182,17 @@ class Component(CaselessDict):
     multiple = ()   # may occur more than once
     exclusive = ()  # These properties are mutually exclusive
     inclusive = ()  # if any occurs the other(s) MUST occur ('duration', 'repeat')
+    ignore_exceptions = False   # if True, and we cannot parse this
+                                # component, we will silently ignore
+                                # it, rather than let the exception
+                                # propagate upwards
 
     def __init__(self, *args, **kwargs):
         "Set keys to upper for initial dict"
         CaselessDict.__init__(self, *args, **kwargs)
         # set parameters here for properties that use non-default values
         self.subcomponents = [] # Components can be nested.
+        self.is_broken = False  # True iff we ignored an exception while parsing a property
 
 
 #    def non_complience(self, warnings=0):
@@ -361,18 +371,30 @@ class Component(CaselessDict):
                 if not stack: # we are at the end
                     comps.append(component)
                 else:
-                    stack[-1].add_component(component)
+                    if not component.is_broken:
+                        stack[-1].add_component(component)
             # we are adding properties to the current top of the stack
             else:
                 factory = types_factory.for_property(name)
-                vals = factory(factory.from_ical(vals))
-                vals.params = params
-                stack[-1].add(name, vals, encode=0)
+                component = stack[-1]
+                try:
+                    vals = factory(factory.from_ical(vals))
+                except ValueError:
+                    if not component.ignore_exceptions:
+                        raise
+                    component.is_broken = True
+                else:
+                    vals.params = params
+                    component.add(name, vals, encode=0)
+
         if multiple:
             return comps
-        if not len(comps) == 1:
+        if len(comps) > 1:
             raise ValueError('Found multiple components where '
-                             'only one is allowed')
+                             'only one is allowed: {st!r}'.format(**locals()))
+        if len(comps) < 1:
+            raise ValueError('Found no components where '
+                             'exactly one is required: {st!r}'.format(**locals()))
         return comps[0]
     from_string = staticmethod(from_string)
 
@@ -426,7 +448,7 @@ class Event(Component):
         'ATTACH', 'ATTENDEE', 'CATEGORIES', 'COMMENT','CONTACT', 'EXDATE',
         'EXRULE', 'RSTATUS', 'RELATED', 'RESOURCES', 'RDATE', 'RRULE'
     )
-
+    ignore_exceptions = True
 
 
 class Todo(Component):
@@ -515,12 +537,38 @@ class Calendar(Component):
     >>> event.set('dtstart', datetime(2005,4,4,8,0,0))
     >>> cal.add_component(event)
     >>> cal.subcomponents[0].as_string()
-    'BEGIN:VEVENT\\r\\nDTSTART:20050404T080000\\r\\nSUMMARY:Python meeting about calendaring\\r\\nUID:42\\r\\nEND:VEVENT\\r\\n'
+    'BEGIN:VEVENT\\r\\nDTSTART;VALUE=DATE:20050404T080000\\r\\nSUMMARY:Python meeting about calendaring\\r\\nUID:42\\r\\nEND:VEVENT\\r\\n'
 
     Write to disc
     >>> import tempfile, os
     >>> directory = tempfile.mkdtemp()
     >>> open(os.path.join(directory, 'test.ics'), 'wb').write(cal.as_string())
+
+    Parsing a complete calendar from a string will silently ignore bogus events.
+    The bogosity in the following is the third EXDATE: it has an empty DATE.
+    >>> s = '\\r\\n'.join(('BEGIN:VCALENDAR',
+    ...                    'PRODID:-//Google Inc//Google Calendar 70.9054//EN',
+    ...                    'VERSION:2.0',
+    ...                    'CALSCALE:GREGORIAN',
+    ...                    'METHOD:PUBLISH',
+    ...                    'BEGIN:VEVENT',
+    ...                    'DESCRIPTION:Perfectly OK event',
+    ...                    'DTSTART;VALUE=DATE:20080303',
+    ...                    'DTEND;VALUE=DATE:20080304',
+    ...                    'RRULE:FREQ=DAILY;UNTIL=20080323T235959Z',
+    ...                    'EXDATE;VALUE=DATE:20080311',
+    ...                    'END:VEVENT',
+    ...                    'BEGIN:VEVENT',
+    ...                    'DESCRIPTION:Bogus event',
+    ...                    'DTSTART;VALUE=DATE:20080303',
+    ...                    'DTEND;VALUE=DATE:20080304',
+    ...                    'RRULE:FREQ=DAILY;UNTIL=20080323T235959Z',
+    ...                    'EXDATE;VALUE=DATE:20080311',
+    ...                    'EXDATE;VALUE=DATE:',
+    ...                    'END:VEVENT',
+    ...                    'END:VCALENDAR'))
+    >>> [e['DESCRIPTION'].ical() for e in Calendar.from_string(s).walk('VEVENT')]
+    ['Perfectly OK event']
     """
 
     name = 'VCALENDAR'
