@@ -206,10 +206,11 @@ class Parameters(CaselessDict):
 
     def from_string(st, strict=False):
         "Parses the parameter format from ical text format"
-        try:
-            # parse into strings
-            result = Parameters()
-            for param in q_split(st, ';'):
+
+        # parse into strings
+        result = Parameters()
+        for param in q_split(st, ';'):
+            try:
                 key, val =  q_split(param, '=')
                 validate_token(key)
                 param_values = [v for v in q_split(val, ',')]
@@ -234,9 +235,9 @@ class Parameters(CaselessDict):
                         result[key] = vals[0]
                     else:
                         result[key] = vals
-            return result
-        except:
-            raise ValueError, 'Not a valid parameter string'
+            except ValueError, e:
+                raise ValueError, '%r is not a valid parameter string: %s' % (param, e)
+        return result
     from_string = staticmethod(from_string)
 
 
@@ -272,6 +273,11 @@ class Contentline(str):
     >>> '\xc3\xab' in str(c)
     True
 
+    Another test of the above
+    >>> c = Contentline('x' * 73 + '\xc3\xab' + '\\n ' + 'y' * 10)
+    >>> str(c).count('\xc3')
+    1
+
     Don't fail if we fold a line that is exactly X times 74 characters long:
     >>> c = str(Contentline(''.join(['x']*148)))
 
@@ -326,17 +332,17 @@ class Contentline(str):
 
     And the traditional failure
     >>> c = Contentline('ATTENDEE;maxm@example.com')
-    >>> c.parts()
+    >>> c.parts()                                               #doctest: +ELLIPSIS
     Traceback (most recent call last):
         ...
-    ValueError: Content line could not be parsed into parts
+    ValueError: Content line could not be parsed into parts...
 
     Another failure:
     >>> c = Contentline(':maxm@example.com')
-    >>> c.parts()
+    >>> c.parts()                                               #doctest: +ELLIPSIS
     Traceback (most recent call last):
         ...
-    ValueError: Content line could not be parsed into parts
+    ValueError: Content line could not be parsed into parts...
 
     >>> c = Contentline('key;param=:value')
     >>> c.parts()
@@ -348,10 +354,10 @@ class Contentline(str):
 
     Should bomb on missing param:
     >>> c = Contentline.from_string("k;:no param")
-    >>> c.parts()
+    >>> c.parts()                                               #doctest: +ELLIPSIS
     Traceback (most recent call last):
         ...
-    ValueError: Content line could not be parsed into parts
+    ValueError: Content line could not be parsed into parts...
 
     >>> c = Contentline('key;param=pvalue:value', strict=False)
     >>> c.parts()
@@ -416,8 +422,8 @@ class Contentline(str):
                                             strict=self.strict)
             values = self[value_split+1:]
             return (name, params, values)
-        except:
-            raise ValueError, 'Content line could not be parsed into parts'
+        except ValueError, e:
+            raise ValueError, "Content line could not be parsed into parts: %r: %s" % (self, e)
 
     def from_string(st, strict=False):
         "Unfolds the content lines in an iCalendar into long content lines"
@@ -456,6 +462,8 @@ class Contentline(str):
                     else:
                         end -= 1
 
+            # Recompute slice, since start or end may have changed.
+            slice = self[start:end]
             new_lines.append(slice)
             if end == l_line:
                 # Done
