@@ -929,7 +929,7 @@ class vRecur(CaselessDict):
     >>> r['byminute'] = 30
     >>> r = vRecur(r)
     >>> r.ical()
-    'BYHOUR=8,9;BYDAY=SU;BYMINUTE=30;BYMONTH=1;FREQ=YEARLY;INTERVAL=2'
+    'FREQ=YEARLY;INTERVAL=2;BYMINUTE=30;BYHOUR=8,9;BYDAY=SU;BYMONTH=1'
 
     >>> r = vRecur(FREQ='yearly', INTERVAL=2)
     >>> r['BYMONTH'] = 1
@@ -937,12 +937,12 @@ class vRecur(CaselessDict):
     >>> r['BYHOUR'] = [8,9]
     >>> r['BYMINUTE'] = 30
     >>> r.ical()
-    'BYDAY=SU;BYMINUTE=30;BYMONTH=1;INTERVAL=2;FREQ=YEARLY;BYHOUR=8,9'
+    'FREQ=YEARLY;INTERVAL=2;BYMINUTE=30;BYHOUR=8,9;BYDAY=SU;BYMONTH=1'
 
     >>> r = vRecur(freq='DAILY', count=10)
     >>> r['bysecond'] = [0, 15, 30, 45]
     >>> r.ical()
-    'COUNT=10;FREQ=DAILY;BYSECOND=0,15,30,45'
+    'FREQ=DAILY;COUNT=10;BYSECOND=0,15,30,45'
 
     >>> r = vRecur(freq='DAILY', until=datetime(2005,1,1,12,0,0))
     >>> r.ical()
@@ -953,23 +953,23 @@ class vRecur(CaselessDict):
     >>> r
     {'COUNT': [10], 'FREQ': ['DAILY'], 'INTERVAL': [2]}
     >>> vRecur(r).ical()
-    'COUNT=10;FREQ=DAILY;INTERVAL=2'
+    'FREQ=DAILY;COUNT=10;INTERVAL=2'
 
     >>> r = vRecur.from_ical('FREQ=YEARLY;INTERVAL=2;BYMONTH=1;BYDAY=-SU;BYHOUR=8,9;BYMINUTE=30')
     >>> r
     {'BYHOUR': [8, 9], 'BYDAY': ['-SU'], 'BYMINUTE': [30], 'BYMONTH': [1], 'FREQ': ['YEARLY'], 'INTERVAL': [2]}
     >>> vRecur(r).ical()
-    'BYDAY=-SU;BYMINUTE=30;INTERVAL=2;BYMONTH=1;FREQ=YEARLY;BYHOUR=8,9'
+    'FREQ=YEARLY;INTERVAL=2;BYMINUTE=30;BYHOUR=8,9;BYDAY=-SU;BYMONTH=1'
 
     Some examples from the spec
 
     >>> r = vRecur.from_ical('FREQ=MONTHLY;BYDAY=MO,TU,WE,TH,FR;BYSETPOS=-1')
     >>> vRecur(r).ical()
-    'BYSETPOS=-1;FREQ=MONTHLY;BYDAY=MO,TU,WE,TH,FR'
+    'FREQ=MONTHLY;BYDAY=MO,TU,WE,TH,FR;BYSETPOS=-1'
 
     >>> r = vRecur.from_ical('FREQ=YEARLY;INTERVAL=2;BYMONTH=1;BYDAY=SU;BYHOUR=8,9;BYMINUTE=30')
     >>> vRecur(r).ical()
-    'BYDAY=SU;BYMINUTE=30;INTERVAL=2;BYMONTH=1;FREQ=YEARLY;BYHOUR=8,9'
+    'FREQ=YEARLY;INTERVAL=2;BYMINUTE=30;BYHOUR=8,9;BYDAY=SU;BYMONTH=1'
 
     and some errors
     >>> r = vRecur.from_ical('BYDAY=12')
@@ -982,6 +982,10 @@ class vRecur(CaselessDict):
     frequencies = ["SECONDLY",  "MINUTELY", "HOURLY", "DAILY", "WEEKLY",
                    "MONTHLY", "YEARLY"]
 
+    canonical_order = [ "FREQ", "UNTIL", "COUNT", "INTERVAL",
+                        "BYSECOND", "BYMINUTE", "BYHOUR", "BYDAY", "BYMONTHDAY", "BYYEARDAY",
+                        "BYWEEKNO", "BYMONTH", "BYSETPOS", "WKST" ]
+
     types = CaselessDict({
         'COUNT':vInt,
         'INTERVAL':vInt,
@@ -1005,7 +1009,7 @@ class vRecur(CaselessDict):
     def ical(self):
         # SequenceTypes
         result = []
-        for key, vals in self.items():
+        for key, vals in self.sorted_items():
             typ = self.types[key]
             if not type(vals) in SequenceTypes:
                 vals = [vals]
@@ -1013,6 +1017,11 @@ class vRecur(CaselessDict):
             result.append('%s=%s' % (key, vals))
         return ';'.join(result)
 
+    def sorted_items(self):
+        """Mac iCal ignores RRULEs where FREQ is not the first rule part.
+        Sorts parts according to the order listed in RFC 5545, section 3.3.10."""
+        return [(k, self[k]) for k in self.canonical_order if self.get(k)]
+
     def parse_type(key, values):
         # integers
         parser = vRecur.types.get(key, vText)
@@ -1071,6 +1080,15 @@ class vText(unicode):
     >>> print vText.from_ical('A string with\\; some\\\\ characters in\\Nit')
     A string with; some\\ characters in
     it
+
+    We are forgiving to UTF8 encoding errors:
+    >>> # We intentionally use a string with unexpected encoding
+    >>> t = vText.from_ical('Ol\\xe9')
+    >>> t
+    u'Ol\\ufffd'
+
+    Notice how accented E character, encoded with latin-1, got replaced
+    with the official U+FFFD REPLACEMENT CHARACTER.
     """
 
     encoding = 'utf-8'
@@ -1107,7 +1125,7 @@ class vText(unicode):
                         .replace(r'\,', ',')
                         .replace(r'\;', ';')
                         .replace('\\\\', '\\'))
-            return ical.decode(vText.encoding)
+            return ical.decode(vText.encoding, 'replace')
         except:
             raise ValueError, 'Expected ical text, got: %s' % ical
     from_ical = staticmethod(from_ical)
@@ -1197,8 +1215,8 @@ class vGeo:
     '1.2;3.0'
 
     >>> g = vGeo.from_ical('37.386013;-122.082932')
-    >>> g
-    (37.386012999999998, -122.082932)
+    >>> g == (float('37.386013'), float('-122.082932'))
+    True
 
     >>> vGeo(g).ical()
     '37.386013;-122.082932'
@@ -1251,7 +1269,7 @@ class vUTCOffset:
 
     >>> u = vUTCOffset(timedelta())
     >>> u.ical()
-    '0000'
+    '+0000'
 
     >>> u = vUTCOffset(timedelta(minutes=-30))
     >>> u.ical()
@@ -1304,7 +1322,7 @@ class vUTCOffset:
         seconds_in_minutes = td.seconds // 60
         total_minutes = day_in_minutes + seconds_in_minutes
         if total_minutes == 0:
-            sign = '%s'
+            sign = '+%s' # Google Calendar rejects '0000' but accepts '+0000'
         elif total_minutes < 0:
             sign = '-%s'
         else:
